import { parse, URLSearchParams } from 'url';
import render_page from './page';
import render_route from './route';
import { ClientManifest, PageComponentManifest, Query, RouteManifest } from '../types';

function parse_querystring(querystring: string): Query {
	const query = {};
	new URLSearchParams(querystring).forEach((value, key) => {
		query[key] = value || true;
	});
	return query;
}

export async function render({
	static_dir,
	template,
	manifest,
	client,
	host,
	url,
	dev,
	App,
	load_component,
	load_route
}: {
	static_dir: string;
	template: string;
	manifest: RouteManifest;
	client: ClientManifest;
	host: string;
	url: string;
	dev: boolean;
	// TODO replace `any`
	App: any; // TODO
	load_component: (component: PageComponentManifest) => Promise<any>;
	load_route: (route: PageComponentManifest) => Promise<any>;
}) {
	const parsed = parse(url);
	const path = parsed.pathname;
	const query = parse_querystring(parsed.query);

	const page = manifest.pages.find(page => page.pattern.test(path));
	const route = manifest.server_routes.find(route => route.pattern.test(path));

	if (route) {
		return render_route({ method: 'GET' /* TODO */, host, path, query, route, load: load_route });
	}

	return render_page({ static_dir, template, manifest, client, host, path, query, page, App, load_component, load_route, dev });
}